#include "curl_asio.hpp"

#include <boost/thread.hpp>
#include <boost/filesystem.hpp>
#include <boost/enable_shared_from_this.hpp>

class Service : public boost::enable_shared_from_this<Service>, private boost::noncopyable
{
public:
  Service(void);
  ~Service(void);

  void test();

protected:
  void startTransfer(std::string & url, boost::filesystem::path & file);

  void workerThread(void);

  void onTransferDone(curl_asio::transfer::ptr transfer, boost::shared_ptr<std::ostream> out, CURLcode result);
  curl_asio::data_action::type onTransferDataRead(boost::shared_ptr<std::ostream> out, const boost::asio::const_buffer& buffer);

private:
  typedef boost::mutex                              Mutex;
  typedef boost::mutex::scoped_lock                 ScopedLock;
  typedef boost::condition_variable                 Condition;

  boost::asio::io_service                           _ioService;
  boost::shared_ptr<boost::asio::io_service::work>  _ioServiceWork;

  Mutex                                             _mutex;
  boost::shared_ptr<boost::thread>                  _thread;
  Condition                                         _condition;
  bool                                              _waitForTransfers;
  boost::shared_ptr<curl_asio>                      _curl;
  std::set<curl_asio::transfer::ptr>                _transfers;
};
